@@ -22,12 +22,12 @@ from threading import current_thread
 from gettext import gettext as _
 from pickle import dump, load
 from urllib.parse import urlparse
-from getpass import getuser
 from time import time
+from getpass import getuser
 import json
 from signal import signal, SIGINT, SIGTERM
 
-from eolie.settings import Settings, SettingsDialog
+from eolie.settings import Settings
 from eolie.window import Window
 from eolie.art import Art
 from eolie.database_history import DatabaseHistory
@@ -37,13 +37,14 @@ from eolie.database_exceptions import DatabaseExceptions
 from eolie.database_settings import DatabaseSettings
 from eolie.database_phishing import DatabasePhishing
 from eolie.sqlcursor import SqlCursor
+from eolie.context import Context
 from eolie.search import Search
 from eolie.download_manager import DownloadManager
 from eolie.menu_pages import PagesMenu
 from eolie.helper_dbus import DBusHelper
 from eolie.helper_task import TaskHelper
 from eolie.define import EOLIE_DATA_PATH, TimeSpan, TimeSpanValues, LoadingType
-from eolie.utils import is_unity, wanted_loading_type, set_proxy_from_gnome
+from eolie.utils import is_unity, wanted_loading_type
 from eolie.logger import Logger
 
 
@@ -62,20 +63,21 @@ class Application(Gtk.Application):
         """
         self.__version = version
         self.__state_cache = []
+        self.__pinned = []
+        self.__ephemeral_context = None
         signal(SIGINT, lambda a, b: self.quit())
         signal(SIGTERM, lambda a, b: self.quit())
         # Set main thread name
         # We force it to current python 3.6 name, to be sure in case of
         # change in python
         current_thread().setName("MainThread")
-        set_proxy_from_gnome()
         # First check WebKit2 version
-        if WebKit2.MINOR_VERSION < 18:
-            exit("You need WebKit2GTK >= 2.18")
+        if WebKit2.MINOR_VERSION < 20:
+            exit("You need WebKit2GTK >= 2.20")
         Gtk.Application.__init__(
-                            self,
-                            application_id="org.gnome.Eolie",
-                            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
+            self,
+            application_id="org.gnome.Eolie",
+            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
         self.set_property("register-session", True)
         # Fix proxy for python
         proxy = GLib.environ_getenv(GLib.get_environ(), "all_proxy")
@@ -120,51 +122,17 @@ class Application(Gtk.Application):
         if self.get_is_remote():
             Gdk.notify_startup_complete()
 
-    def get_app_menu(self):
-        """
-            Setup application menu
-            @return menu as Gio.Menu
-        """
-        builder = Gtk.Builder()
-        builder.add_from_resource("/org/gnome/Eolie/Appmenu.ui")
-        menu = builder.get_object("app-menu")
-
-        settings_action = Gio.SimpleAction.new("settings", None)
-        settings_action.connect("activate", self.__on_settings_activate)
-        self.add_action(settings_action)
-
-        about_action = Gio.SimpleAction.new("about", None)
-        about_action.connect("activate", self.__on_about_activate)
-        self.add_action(about_action)
-
-        show_sidebar = self.settings.get_value("show-sidebar")
-        sidebar_action = Gio.SimpleAction.new_stateful(
-                                       "sidebar",
-                                       None,
-                                       GLib.Variant.new_boolean(show_sidebar))
-        sidebar_action.connect("change-state", self.__on_sidebar_change_state)
-        self.add_action(sidebar_action)
-
-        shortcuts_action = Gio.SimpleAction.new("shortcuts", None)
-        shortcuts_action.connect("activate", self.__on_shortcuts_activate)
-        self.add_action(shortcuts_action)
-
-        quit_action = Gio.SimpleAction.new("quit", None)
-        quit_action.connect("activate", lambda x, y: self.quit())
-        self.add_action(quit_action)
-        return menu
-
     def update_default_style_sheet(self):
         """
             Should be called on startup
         """
         rules = self.adblock.get_default_css_rules()
         self.__default_style_sheet = WebKit2.UserStyleSheet(
-                                 rules,
-                                 WebKit2.UserContentInjectedFrames.ALL_FRAMES,
-                                 WebKit2.UserStyleLevel.USER,
-                                 None,
-                                 None)
+            rules,
+            WebKit2.UserContentInjectedFrames.ALL_FRAMES,
+            WebKit2.UserStyleLevel.USER,
+            None,
+            None)
 
     def do_startup(self):
         """
@@ -182,7 +150,7 @@ class Application(Gtk.Application):
         windows = self.get_windows()
         if windows and size is None:
             active_window = self.active_window
-            size = active_window.get_size()
+            size = active_window.size
             maximized = active_window.is_maximized()
         window = Window(self, size, maximized)
         window.connect('delete-event', self.__on_delete_event)
@@ -215,6 +183,22 @@ class Application(Gtk.Application):
                 self.__unity.set_property("progress", fraction)
                 self.__unity.set_property("progress_visible", fraction != 1.0)
 
+    def add_to_pinned(self, uri):
+        """
+            Add uri to pinned pages
+            @param uri as str
+        """
+        if uri not in self.__pinned:
+            self.__pinned.append(uri)
+
+    def remove_from_pinned(self, uri):
+        """
+            Remove uri from pinned pages
+            @param uri as str
+        """
+        if uri in self.__pinned:
+            self.__pinned.remove(uri)
+
     def set_profiles(self):
         """
             Set profiles
@@ -258,7 +242,7 @@ class Application(Gtk.Application):
         if active_id != TimeSpan.FOREVER:
             atime = time()
             if active_id != TimeSpan.NEVER:
-                atime -= TimeSpanValues[active_id]/1000000
+                atime -= TimeSpanValues[active_id] / 1000000
             self.history.clear_to(int(atime))
 
         if self.sync_worker is not None:
@@ -272,6 +256,14 @@ class Application(Gtk.Application):
         else:
             Gio.Application.quit(self)
 
+    @property
+    def pinned(self):
+        """
+            Get pinned pages
+            @return [str]
+        """
+        return self.__pinned
+
     @property
     def profiles(self):
         """
@@ -295,6 +287,17 @@ class Application(Gtk.Application):
             value = self.search.uri
         return value
 
+    @property
+    def ephemeral_context(self):
+        """
+            Return ephemeral context
+            @return WebKit2.WebContext
+        """
+        if self.__ephemeral_context is None:
+            self.__ephemeral_context = WebKit2.WebContext.new_ephemeral()
+            Context(self.__ephemeral_context)
+        return self.__ephemeral_context
+
     @property
     def default_style_sheet(self):
         """
@@ -319,13 +322,6 @@ class Application(Gtk.Application):
         """
         return self.get_windows()
 
-    @property
-    def favicons_path(self):
-        """
-            Cookies sqlite DB path
-        """
-        return self.__FAVICONS_PATH
-
     @property
     def extension_dir(self):
         """
@@ -334,6 +330,13 @@ class Application(Gtk.Application):
         """
         return self.__extension_dir
 
+    @property
+    def favicons_path(self):
+        """
+            Cookies sqlite DB path
+        """
+        return self.__FAVICONS_PATH
+
 #######################
 # PRIVATE             #
 #######################
@@ -350,24 +353,17 @@ class Application(Gtk.Application):
             new_path = new_path + ':' + current_path
         GLib.setenv("PYTHONPATH", new_path, True)
 
-        # Create favicon path
-        if not GLib.file_test(self.__FAVICONS_PATH, GLib.FileTest.IS_DIR):
-            GLib.mkdir_with_parents(self.__FAVICONS_PATH, 0o0750)
-
         # Add a global DBus helper
         self.helper = DBusHelper()
         # First init sync worker
-        from eolie.mozilla_sync import SyncWorker
+        from eolie.firefox_sync import SyncWorker
         if SyncWorker.check_modules():
             self.sync_worker = SyncWorker()
             self.sync_worker.sync_loop()
         else:
             self.sync_worker = None
-        if self.prefers_app_menu():
-            menu = self.get_app_menu()
-            self.set_app_menu(menu)
         cssProviderFile = Gio.File.new_for_uri(
-                'resource:///org/gnome/Eolie/application.css')
+            'resource:///org/gnome/Eolie/application.css')
         cssProvider = Gtk.CssProvider()
         cssProvider.load_from_file(cssProviderFile)
         screen = Gdk.Screen.get_default()
@@ -413,12 +409,14 @@ class Application(Gtk.Application):
                 gi.require_version('Unity', '7.0')
                 from gi.repository import Unity
                 self.__unity = Unity.LauncherEntry.get_for_desktop_id(
-                                                     "org.gnome.Eolie.desktop")
+                    "org.gnome.Eolie.desktop")
             except:
                 pass
 
         # Init profiles
         self.set_profiles()
+        # Init default context
+        Context(WebKit2.WebContext().get_default())
 
         shortcut_action = Gio.SimpleAction.new('shortcut',
                                                GLib.VariantType.new('s'))
@@ -427,7 +425,7 @@ class Application(Gtk.Application):
         self.set_accels_for_action("win.shortcut::expose", ["<Alt>e"])
         self.set_accels_for_action("win.exceptions::site", ["<Control>e"])
         self.set_accels_for_action("win.shortcut::jsblock", ["<Control>j"])
-        self.set_accels_for_action("win.shortcut::show_left_panel", ["F9"])
+        self.set_accels_for_action("win.shortcut::show_sidebar", ["F9"])
         self.set_accels_for_action("win.shortcut::uri",
                                    ["<Control>l", "<Control>b"])
         self.set_accels_for_action("win.shortcut::new_page", ["<Control>t"])
@@ -523,7 +521,7 @@ class Application(Gtk.Application):
 
         window_state = {}
         window_state["id"] = str(window)
-        window_state["size"] = window.get_size()
+        window_state["size"] = window.size
         window_state["maximized"] = window.is_maximized()
         session_states = []
         if self.settings.get_value("remember-session"):
@@ -540,6 +538,7 @@ class Application(Gtk.Application):
                 if state is not None:
                     session_states.append(state)
         window_state["states"] = session_states
+        window_state["sites"] = window.container.sites_manager.sort
         return window_state
 
     def __save_state(self):
@@ -555,6 +554,8 @@ class Application(Gtk.Application):
                 window_states.append(window_state)
             dump(window_states,
                  open(EOLIE_DATA_PATH + "/session_states.bin", "wb"))
+            dump(self.__pinned,
+                 open(EOLIE_DATA_PATH + "/pinned.bin", "wb"))
         except Exception as e:
             Logger.error("Application::__save_state(): %s", e)
 
@@ -575,32 +576,63 @@ class Application(Gtk.Application):
         """
         size = (800, 600)
         maximized = False
+        pinned = []
         try:
-            windows = load(open(EOLIE_DATA_PATH + "/session_states.bin", "rb"))
+            pinned = load(
+                open(EOLIE_DATA_PATH + "/pinned.bin", "rb"))
+            window_states = load(
+                open(EOLIE_DATA_PATH + "/session_states.bin", "rb"))
             if self.settings.get_value("remember-session"):
-                for window in windows:
-                    if not window["states"]:
-                        continue
-                    new_window = self.get_new_window(window["size"],
-                                                     window["maximized"])
+                for window_state in window_states:
+                    window = self.get_new_window(window_state["size"],
+                                                 window_state["maximized"])
+                    window.container.sites_manager.set_initial_sort(
+                        window_state["sites"])
                     items = []
                     i = 0 if foreground else 1
                     for (uri, title, atime,
-                         ephemeral, state) in window["states"]:
+                         ephemeral, state) in window_state["states"]:
+                        if uri in pinned:
+                            pinned.remove(uri)
                         loading_type = wanted_loading_type(i)
                         webkit_state = WebKit2.WebViewSessionState(
-                                                         GLib.Bytes.new(state))
+                            GLib.Bytes.new(state))
                         items.append((uri, title, atime, ephemeral,
                                       webkit_state, loading_type))
                         i += 1
-                    new_window.container.add_webviews(items)
-            elif windows:
-                    size = windows[0]["size"]
-                    maximized = windows[0]["maximized"]
+                    if window_state["states"]:
+                        window.container.add_webviews(items)
+                    else:
+                        window.container.add_webview(
+                            self.start_page,
+                            LoadingType.FOREGROUND,
+                            False)
+            elif window_states:
+                size = window_states[0]["size"]
+                maximized = window_states[0]["maximized"]
         except Exception as e:
             Logger.error("Application::__create_initial_windows(): %s", e)
-        if not self.get_windows():
-            self.get_new_window(size, maximized)
+        # Create a window if None available and setup pinned pages
+        # We already removed saved states from pinned
+        windows = self.get_windows()
+        if windows:
+            window = windows[0]
+        else:
+            window = self.get_new_window(size, maximized)
+            window.container.add_webview(
+                            self.start_page,
+                            LoadingType.FOREGROUND,
+                            False)
+        for uri in pinned:
+            window.container.add_webview(uri,
+                                         LoadingType.BACKGROUND,
+                                         False)
+        # Really setup pinned
+        try:
+            self.__pinned = load(
+                    open(EOLIE_DATA_PATH + "/pinned.bin", "rb"))
+        except:
+            self.__pinned = []
 
     def __on_handle_local_options(self, app, options):
         """
@@ -650,7 +682,9 @@ class Application(Gtk.Application):
                 items.append((uri, uri, 0, ephemeral, None, loading_type))
                 i += 1
             active_window.container.add_webviews(items)
-            active_window.present()
+            # https://bugzilla.gnome.org/show_bug.cgi?id=766284
+            monotonic_time = int(GLib.get_monotonic_time() / 1000)
+            active_window.present_with_time(monotonic_time)
         # Add default start page
         if not active_window.container.views:
             active_window.container.add_webview(self.start_page,
@@ -772,7 +806,7 @@ class Application(Gtk.Application):
             close = builder.get_object("close")
             cancel = builder.get_object("cancel")
             label.set_text(_("Downloads running,"
-                             " are you sure you want quit ?"))
+                             " are you sure you want quit?"))
             dialog.set_transient_for(window)
             dialog.connect("response", on_response_id, window)
             close.connect("clicked", on_close, dialog)
@@ -782,68 +816,15 @@ class Application(Gtk.Application):
             self.__try_closing(window, window.container.views)
         return True
 
-    def __on_settings_activate(self, action, param):
-        """
-            Show settings dialog
-            @param action as Gio.SimpleAction
-            @param param as GLib.Variant
-        """
-        dialog = SettingsDialog(self.active_window)
-        dialog.show()
-
-    def __on_about_activate(self, action, param):
-        """
-            Setup about dialog
-            @param action as Gio.SimpleAction
-            @param param as GLib.Variant
-        """
-        builder = Gtk.Builder()
-        builder.add_from_resource('/org/gnome/Eolie/AboutDialog.ui')
-        about = builder.get_object('about_dialog')
-        window = self.active_window
-        if window is not None:
-            about.set_transient_for(window)
-        about.connect("response", self.__on_about_activate_response)
-        about.show()
-
-    def __on_shortcuts_activate(self, action, param):
-        """
-            Show help in yelp
-            @param action as Gio.SimpleAction
-            @param param as GLib.Variant
-        """
-        builder = Gtk.Builder()
-        builder.add_from_resource("/org/gnome/Eolie/Shortcuts.ui")
-        shortcuts = builder.get_object("shortcuts")
-        window = self.active_window
-        if window is not None:
-            shortcuts.set_transient_for(window)
-        shortcuts.show()
-
-    def __on_about_activate_response(self, dialog, response_id):
-        """
-            Destroy about dialog when closed
-            @param dialog as Gtk.Dialog
-            @param response ID as int
-        """
-        dialog.destroy()
-
-    def __on_sidebar_change_state(self, action, value):
-        """
-            Show/hide sidebar
-            @param action as Gio.SimpleAction
-            @param value as bool
-        """
-        action.set_state(value)
-        self.settings.set_value("show-sidebar", GLib.Variant("b", value))
-
     def __on_activate(self, application):
         """
             Call default handler, raise last window
             @param application as Gio.Application
         """
         if self.get_windows():
-            self.active_window.present()
+            # https://bugzilla.gnome.org/show_bug.cgi?id=766284
+            monotonic_time = int(GLib.get_monotonic_time() / 1000)
+            self.active_window.present_with_time(monotonic_time)
 
     def __on_shortcut_action(self, action, param):
         """
